@@ -254,7 +254,6 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
     hex_digits[256];
 
   unsigned long
-    extent,
     spotcolor;
 
   (void) memset(&bounds,0,sizeof(bounds));
@@ -287,7 +286,6 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
   hex_digits[(int) 'D']=13;
   hex_digits[(int) 'E']=14;
   hex_digits[(int) 'F']=15;
-  extent=0;
   priority=0;
   *version='\0';
   spotcolor=0;
@@ -413,10 +411,14 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
         unsigned char
           *q;
 
+        unsigned long
+          extent;
+
         /*
           Read Photoshop profile.
         */
         p=GetMagickByteBufferDatum(&buffer);
+        extent=0;
         count=(ssize_t) sscanf(p,PhotoshopProfile " %lu",&extent);
         if ((count != 1) || (extent == 0))
           continue;
@@ -431,7 +433,7 @@ static void ReadPSInfo(const ImageInfo *image_info,Image *image,
           if (c == EOF)
             break;
           *q++=(unsigned char) c;
-          extent-=2;
+          extent-=MagickMin(extent,2);
         }
         SetStringInfoLength(ps_info->photoshop_profile,length);
         continue;
